//////////////////////////////////////////////////////////////

var video_mode = 7; // 800x600
var video_depth = 16; // 16 bit mode

var offset_right = 7; // distance between the player and the rocket starting position on x
var offset_front = 40; // on y
var offset_up = -5; // and on z
var offset_angle = -30; // angle between the player and the rocket at starting point
// play with the 4 values above 

var temp_target = 300; // temporary target placed 300 quants in front of the player

var player_speed;
var rocket_speed;
var bullet_speed;
var recoil = 0;
var rocket_pos;

var chopper_index[50]; // up to 50 choppers
var temp_counter = 0; // array index
var distance = 20000; // default distance between the chopper and the player at game start
var found_chopper = 1; // index for the closest chopper
var chopper_number = 0; // will be set to 1 by the first chopper, to 2 by the 2nd chopper and so on
var remaining_choppers = 0; 

//////////////////////////////////////////////////////////////

font univers_font =  <univers.pcx>, 16, 19;

//////////////////////////////////////////////////////////////

define health skill30;

//////////////////////////////////////////////////////////////

bmap target_pcx = <target.pcx>;
bmap aiming_pcx = <aiming.pcx>;
bmap smoke_pcx = <smoke.pcx>;

//////////////////////////////////////////////////////////////

sound gotgun_snd = <gotgun.wav>;
sound shoot_snd = <shoot.wav>;
sound chopperx_snd = <chopperx.wav>;
sound playerhit_snd = <playerhit.wav>;

sound beep_sound = <beep.wav>;

//////////////////////////////////////////////////////////////

string aircombat_wmb = <aircombat.wmb>;

string rocket_mdl = <rocket.mdl>;
string chopper_mdl = <chopper.mdl>;
string explo13_pcx = <explo+13.pcx>;
string bullet_pcx = <bullet.pcx>;

string status_str = "Health";

//////////////////////////////////////////////////////////////

function game_init();
function player_event();
function move_bullet();
function remove_me();
function create_smoke();
function fade_smoke();
function move_chopper();
function chopper_bullet();	
function chopper_event();
function explode_me();

//////////////////////////////////////////////////////////////

panel target_pan // split this panel in 4 smaller pieces to save lots of video memory
{
	bmap = target_pcx;
	layer = 20;	
	pos_x = 0;
	pos_y = 0;
	flags = transparent, overlay, refresh, d3d, visible;
}

panel aiming_pan // split this panel in 4 smaller pieces to save lots of video memory
{
	bmap = aiming_pcx;
	layer = 20;	
	pos_x = 400;
	pos_y = 280;
	flags = overlay, refresh, d3d;
}

panel status_pan // displays the health and the number of enemies that were shot
{
	pos_x = 0;
	pos_y = 0;
	digits = 10, 30, 4, univers_font, 1, player.health;
	flags = d3d, refresh, visible;
}

text status_txt // displays the text
{
	pos_x = 0;
	pos_y = 0;
	font = univers_font;	
	string = status_str;
	flags = visible;
}


//////////////////////////////////////////////////////////////

function main()
{
	sky_clip = -90; // renders the whole sky
	on_d = null; // disables the debug panel
	level_load (aircombat_wmb);
	wait (2);
	game_init(); 
}	

action player_moves
{
	player = me;
	my.health = 100;
	my.invisible = on; // no need to see the player because we are in 1st person mode

	my.enable_impact = on;
	my.enable_entity = on;
	my.event = player_event;

	while (my.health > 0)
	{
		vec_set (camera.pos, my.pos);
		camera.tilt += 20 * mouse_force.y * time;
		camera.pan -= 20 * mouse_force.x * time;
		my.pan = camera.pan;	
		my.tilt = camera.tilt;
		player_speed.x = 15 * (key_w - key_s) * time - 0.5 * recoil; // move forward / backward
		player_speed.y = 10 * (key_a - key_d) * time; // strafe left / right
		vec_set (temp, my.x);
		temp.z -= 1000;
		trace_mode = ignore_me + use_box;
		player_speed.z = -trace (my.x, temp);					
		move_mode = ignore_you + ignore_passable;	
		ent_move(player_speed, nullvector);

		wait (1);
	}
}

function player_event()
{
	wait (1);
	exclusive_global;
	if (my.health > 0)
	{
		my.health -= 0.5;
		snd_play (playerhit_snd, 40, 0);
	}
	else
	{
		my.health = 0;
	}
}


function game_init() 
{
	while (player == null) {wait (1);}
	while (1)
	{
		if ((mouse_left == 1) && (player.health > 0)) // fire
		{
			snd_play (shoot_snd, 70, 0);
			ent_create (rocket_mdl, player.pos, move_bullet);
			recoil = 1;
			waitt (2);
			recoil = 0;
			while (mouse_left == 1) {wait (1);} // disable autofire		
		}

		temp_counter += 1;
		if (temp_counter == 50) 
		{
			temp_counter -= 49;
		}
		if (chopper_index[temp_counter] < distance && chopper_index[temp_counter] > 0)
		{
			distance = chopper_index[temp_counter];
			found_chopper = temp_counter;		
		}
		else
		{
			distance = chopper_index[found_chopper];
		}
		if (remaining_choppers == 0) 
		{
			aiming_pan.visible = off; // hide the aiming panel if all the choppers are dead
		}

		wait (1);
	}			
}

function move_bullet()
{
	my.passable = on;
	my.enable_entity = on;
	my.enable_block = on;
	my.event = remove_me;

	my.pan = camera.pan;
	my.tilt = camera.tilt;

	my.skill1 = 0;
	rocket_speed.x = 500 * time;
	rocket_speed.y = 0;
	rocket_speed.z = 0;
	while (my.skill1 < 100)
	{
		if (vec_dist (my.x, player.x) > 50)
		{
			my.passable = off;
		}
		my.roll += 25 * time;
		my.skill1 += 1 * time;
		effect (create_smoke, 1, my.x, normal);
		move_mode = ignore_you + ignore_passable;
		ent_move (rocket_speed, nullvector);
		wait (1);
	}
	ent_remove (me); // if it has travelled too much without hitting anything
}

function remove_me()
{
	waitt (2);
	ent_remove (me);
}

function create_smoke
{
	temp.x = random(4) - 2;
	temp.y = random(4) - 2;
	temp.z = random(3) + 3;
	vec_add (my.vel_x, temp);
	my.flare = on;
	my.bright = on;
	my.alpha = 50;
	my.bmap = smoke_pcx;
	my.size = 30;
	my.move = on;
	my.lifespan = 100;
	my.function = fade_smoke;
}

function fade_smoke()
{
	my.alpha -= 2 * time;
	my.size -= 2 * time;
	if ((my.alpha < 0) || (my.size < 2)) {my.lifespan = 0;}
}

action chopper_generator
{
	my.invisible = on;
	my.passable = on;
	waitt (64 + random (64)); // wait for 4..8 seconds
	ent_create (chopper_mdl, my.pos, move_chopper);
}	

// the choppers can die without being hit by player's rockets - I'll let you guess how :)
function move_chopper()
{
	chopper_number += 1;
	remaining_choppers += 1;
	my.skill1 = chopper_number;
	my.enable_impact = on;
	my.enable_block = on;
	my.enable_entity = on;
	my.event = chopper_event;
	
	my.skill40 = 1; // we set skill40 to 1 for all the choppers

	while (my != null)
	{
		vec_for_vertex(my.skill18, my, 55);	// chopper bullet position
		if (my.skill42 == 0 && abs(my.x + my.y - player.x - player.y) > 2000) // return if the distance to the player is above 2000 quants
		{
			vec_set (temp.x, player.x); 
			vec_sub (temp.x, my.x);
			vec_to_angle (my.pan, temp); // turn towards the player	
		}
		my.tilt = 0;

		ent_cycle("fly", my.skill10); 
		my.skill10 += my.skill11 * time;
		if (my.skill11 < 10) 
		{
			my.skill11 += 0.2 * time; // increase the rotation speed
		}	
		if (my.skill10 > 100) 
		{
			my.skill10 = 0; // loop animation
		} 

		if ((my.skill17 < 3 + random(3)) && (my.skill12 == 0)) // random altitude for every chopper
		{
			my.skill15 = 0; // skills 15..17 are used a var here
			my.skill16 = 0;
			my.skill17 += 0.05 * time;			
		}
		else
		{
			my.skill12 = 1;
			if (my.skill15 < 5 * time)
			{
				my.skill15 += 2 * time;
			}
			my.skill16 = 0;
			my.skill17 = 0;
			if (500 < abs(my.x + my.y - player.x - player.y) < 1000) // close to the player -> fire bullets
			{
				my.skill25 += 3 * time; // a simple counter
				my.skill26 = int (my.skill25);
				if ((my.skill26 % 100 == 0) && (vec_dist (player.x, my.x) > vec_dist (player.x, my.skill18)))  // change "100" to adjust the fire rate
				{
					ent_create (bullet_pcx, my.skill18, chopper_bullet); // don't fire until the chopper is up
				}
			}			
		}

		chopper_index[my.skill1] = vec_dist(my.x, player.x);
		if (distance > chopper_index[my.skill1] - 4000) // play with this value - it depends on the size of your level
		{	
			vec_set (temp, my.x);
			if (vec_to_screen (temp, camera)) // if the target is visible
			{
				aiming_pan.pos_x = temp.x - bmap_width(aiming_pcx) / 2;			
				aiming_pan.pos_y = temp.y - bmap_height(aiming_pcx) / 2;
				aiming_pan.visible = on;
			}
			else
			{
				aiming_pan.visible = off;
			}
		}

		move_mode = ignore_you + ignore_passable;	
		ent_move(my.skill15, nullvector);

		wait (1);
	}
}

function chopper_event()
{
	if (((event_type == event_entity) || (event_type == event_impact)) && (you.skill40 == 1))
	{
		my.pan -= 180; // get away
		my.skill42 = 1;
		waitt (64); // move away for 4 seconds
		my.skill42 = 0;
	}
	else
	{
		chopper_index[my.skill1] = 0; // don't use auto-aiming for this chopper from now on
		distance = 20000; 
		remaining_choppers -= 1;
		ent_playsound (my, chopperx_snd, 1000);
		my.passable = on;
		my.ambient = 50;
		waitt (2);
		ent_create (explo13_pcx, my.pos, explode_me);
		waitt (4);
		my.transparent = on;
		ent_create (explo13_pcx, my.pos, explode_me);
		ent_remove (me);
	}
}

function explode_me()
{
	my.oriented = on;
	my.flare = on;
	my.bright = on;
	my.ambient = 100;
	while (my.frame < 14)
	{
		if (my.scale_x < 3)
		{
			my.scale_x += 0.5 * time;
			my.scale_y = my.scale_x;
		}
		my.frame += 1 * time;

		vec_set (temp.x, player.x);
		vec_sub (temp.x, my.x);
		vec_to_angle (my.pan, temp); // turn towards the player to make sure that the explo looks great

		wait (1);
	}
	ent_remove (me);
}

function chopper_bullet()
{
	my.passable = on;
	my.oriented = on;
	my.flare = on;
	my.bright = on;	
	my.scale_x = 0.5;
	my.scale_y = my.scale_x;

	my.enable_entity = on;
	my.enable_block = on;
	my.event = remove_me;

	vec_set (temp.x, player.x); 
	vec_sub (temp.x, my.x);
	vec_to_angle (my.pan, temp); // the bullet moves towards the player	

	my.skill1 = 0;
	bullet_speed.x = 100 * time;
	bullet_speed.y = 0;
	bullet_speed.z = 0;
	while (my != null)
	{
		if (my.roll > 200) {my.passable = off;} // it works :)
		my.roll += 25 * time;
		my.skill1 += 1 * time;
		move_mode = ignore_you + ignore_passable;
		ent_move (bullet_speed, nullvector);
		wait (1);
	}
}